<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<!-- NOTE: don't forget to increase version ID when this file is modified -->
<gui version="6" name="kxmlkipicmd" >

    <MenuBar>

        <Menu name="File"><text>&amp;File</text>
            <Action name="app_exit"/>
        </Menu>

        <Menu name="Plugins"><text>&amp;Plugins</text>

            <Menu name="Image"><text>&amp;Image</text>
                <Action name="emptyCategoryImage" />
                <DefineGroup name="kipi_image_group" append="kipi_image_group" />
            </Menu>

            <Menu name="View"><text>&amp;View</text>
            </Menu>

            <Menu name="Tools"><text>&amp;Tools</text>
                <Action name="emptyCategoryTools" />
                <DefineGroup name="kipi_album_group" append="kipi_album_group" />
                <Separator />
                <DefineGroup name="kipi_tool_group" append="kipi_tool_group" />
                <Separator />
                <DefineGroup name="kipi_batch_group" append="kipi_batch_group" />
            </Menu>

            <Menu name="Import"><text>I&amp;mport</text>
                <Action name="emptyCategoryImport" />
                <DefineGroup name="kipi_import_group" append="kipi_import_group" />
            </Menu>

            <Menu name="Export"><text>&amp;Export</text>
                <Action name="emptyCategoryExport" />
                <DefineGroup name="kipi_export_group" append="kipi_export_group" />
            </Menu>

            <Merge />

        </Menu>

        <Menu name="settings" noMerge="1"><Text>&amp;Settings</Text>
            <Merge name="StandardToolBarMenuHandler" />
            <Action name="options_show_menubar"/>
            <Separator/>
            <Action name="options_configure_keybinding"/>
            <Action name="options_configure_toolbars"/>
            <Action name="options_configure"/>
        </Menu>

    </MenuBar>

    <ToolBar name="mainToolBar"><text>Main Toolbar</text>
        <Action name="app_exit" />
        <Action name="options_show_menubar" />
        <Action name="options_show_toolbar" />
        <Separator/>
        <Action name="options_configure_keybinding" />
        <Action name="options_configure_toolbars" />
        <Merge />
        <WeakSeparator />
        <Action name="options_configure" />
    </ToolBar>

    <ActionProperties/>

</gui>
