<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<!-- NOTE: don't forget to increase version ID when this file is modified -->
<gui version="11" name="kipiplugin_kxmlkipicmd" >

    <MenuBar>

        <Menu name="Image"><text>&amp;Image</text>
            <Action name="kxmlhelloworld-actionImage" group="kipi_image_group" />
        </Menu>

        <Menu name="Tools"><text>&amp;Tools</text>
            <Action name="kxmlhelloworld-actionTools" group="kipi_tool_group" />
        </Menu>

        <Menu name="Import"><text>I&amp;mport</text>
            <Action name="kxmlhelloworld-actionImport" group="kipi_import_group" />
        </Menu>

        <Menu name="Export"><text>&amp;Export</text>
            <Action name="kxmlhelloworld-actionExport" group="kipi_export_group" />
        </Menu>

    </MenuBar>

    <ToolBar name="mainToolBar"><text>Main Toolbar</text>
    </ToolBar>

</gui>
